#!/usr/bin/perl

use strict;

use vars qw( @kb @row %out @roworder %quotations );

{

    my $title = "Happy Hacking Finnish keyboard map for Linux";
    my $email = 'kato-hh@kato.iki.fi';
    my $date = `date '+%X %x'`;
    my $url = 'http://www.iki.fi/kato/kbd-fi-hh/';

    my $top = <<EOT
<HTML><HEAD><TITLE>$title</TITLE>
<STYLE>
    body { background-color: #404060; color: white; font: 12pt sans-serif }
    h1 { font: 18pt sans-serif }
    a { color: yellow }
    p { padding: 0em 2em; color: white; }
    ul { padding: 0em 2em; color: white; font: 10pt sans-serif }
    table { background-color: #E0E0E0; color: black }
    td { color: black }
    td.a { background-color: #E0E0E0; font: 18pt courier }
    td.b { background-color: #D0D0D0; font: 14pt courier }
    td.c { background-color: #C0C0C0; font: 18pt courier }
    td.d { background-color: #B0B0B0; font: 14pt courier }
</STYLE>	    
</HEAD><BODY>

<H1>$title, v0.6</H1>
<A HREF="$url">Hauskaa Hakkailua</A>

    <P><A HREF="http://www.pfuca.com/">Happy Hacking</A> keyboard is
missing a key that holds the &lt;/&gt; characters on a Finnish
keyboard. These keyboard maps for text console and X fix that. I also
tried to make most of the usual characters easily available, by way of
a pretty unstandard mixture of the US and FI keyboards.</P>

<P>Note that a Happy Hacking keyboard seems likely to frustrate any
non-english keymap users. Though, <I>if</I> you are a vi-oriented coder, it
sure saves some physical space. <A
HREF="http://www.nuutti.com/">Nuutti</A> imports these things in Finland.</P>

<P>Happy Hackers with other Scandinavian/European keymaps may find this partially usable. Stay alert.</P>

<P>This stuff is obviously free for anyone to copy/modify/resell. I hope nobody succeeds in patenting or holding copyright for a particular keyboard map anyway :P</P>

<P><A HREF="../kbd-fi-hh.tar.gz">Download package</A> includes installation script for Red Hat 6.1.</P>

<P><CENTER><TABLE BORDER=1 CELLSPACING=1 CELLPADDING=0>
EOT
    ;

my $bottom = <<EOT
</TABLE>
</CENTER>
<UL>
<LI>Two right-topmost keys are mostly "dead", ie. will wait for a letter to modify. 
<LI>"Alt" is (mostly) left-Alt on text console, Alt-Gr in X. You should probably fix that...
<LI>Console version is not exactly same, due to difference in originals.
</UL></P>
<P ALIGN="RIGHT">$date <A HREF="mailto:$email?subject=kbd-fi-hh">$email</A></P>
</BODY></HTML>
EOT
    ;

my $legend = <<EOT
<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=1>
 <TR><TD COLSPAN="2" ALIGN="CENTER">Legend</TD></TR>
 <TR><TD CLASS="b">Shift</TD><TD CLASS="d">Alt<BR>Shift</TD></TR>
 <TR><TD CLASS="a">key</TD><TD CLASS="c">Alt</TD></TR>
</TABLE>
EOT
    ;


%out = (
	'top' => $top,
	'bottom' => $bottom,
	'legend' => "<TD COLSPAN=3 ROWSPAN=2 VALIGN=BOTTOM ALIGN=RIGHT>$legend</TD>",
	'rowbegin' => " <TR>\n",
	'rowend' => " </TR>\n",
	'keybegin' => "  <TD><TABLE BORDER=0 CELLSPACING=1 CELLPADDING=2> ",
	'keyend' => "</TABLE></TD>\n",
	'strokebegin' => [
			  ' <TR><TD CLASS="b">',
			  '<TD CLASS="d">',
			  ' <TR><TD CLASS="a">',
			  '<TD CLASS="c">',
			  ],
	'strokeend' => [
			'</TD>',
			'</TD></TR>',
			'</TD>',
			'</TD></TR> ',
			],

	);

@roworder = (1,3,0,2);
%quotations = ( 
		'<' => 'lt',
		'>' => 'gt',
		);

}       

### MAIN

# IN

my $fullrow=[];
while( defined(my $row = <STDIN>) ) {
    chomp $row;
    if( $row eq '' ) {
	my @fullrow = @$fullrow;
	if( $#fullrow>=0 ) {
	    push @kb, $fullrow;
	    $fullrow=[];
	}
	next;
    }
    my @currow = split('',$row);
    push @$fullrow, \@currow;
}

#use Data::Dumper;
#print Dumper(\@kb);
#exit;

# OUT

print $out{'top'};

my $r=0;
foreach my $row ( @kb ) {
    @row=@$row;
    my @firstrow=@{$row[0]};
    print $out{'rowbegin'};
    for( my $i=0; $i<=$#firstrow; $i++) {
	print STDERR "KEY $row[0]->[$i]\n";
	print $out{'keybegin'};
	my $k=0;
	foreach my $j ( @roworder ) {
	    next if ! $row[$j];
	    my $c = $row[$j]->[$i];
	    $c = '&'.$quotations{$c}.";" if $quotations{$c};
	    print STDERR " - $j=$c\n";
	    print $out{'strokebegin'}->[$k], $c, $out{'strokeend'}->[$k];
	    $k++;
	}
	print $out{'keyend'};
    }
    print $out{'legend'} if $r==2;
    print $out{'rowend'};
    $r++;
}
print $out{'bottom'};
